@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2011-2013 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2011-2014 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -21,7 +21,7 @@ import os
 import posixpath
 import textwrap
 
-VERSION = '3.6'
+VERSION = '3.7'
 ENCODING = 'utf-8'
 PY3 = sys.version_info >= (3,)
 PACKAGE_DIR = os.path.dirname(__file__)
@@ -56,6 +56,8 @@ def wrap(text, width):
 def get_int_param(num_str):
     if num_str.startswith('$'):
         return int(num_str[1:], 16)
+    if num_str.startswith('%'):
+        return int(num_str[1:], 2)
     return int(num_str)
 
 def parse_int(num_str, default=None):
