@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2010-2013 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2010-2014 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -17,7 +17,8 @@
 # SkoolKit. If not, see <http://www.gnu.org/licenses/>.
 
 from . import write_line, parse_int, open_file
-from .skoolparser import DIRECTIVES, get_address, parse_asm_block_directive, get_instruction_ctl, get_defb_length, get_defm_length, get_defw_length
+from .skoolparser import (DIRECTIVES, parse_asm_block_directive, get_instruction_ctl,
+                          get_defb_length, get_defm_length, get_defs_length, get_defw_length)
 
 BLOCKS = 'b'
 BLOCK_TITLES = 't'
@@ -54,8 +55,8 @@ def get_lengths(stmt_lengths):
     return ','.join(length_params)
 
 class CtlWriter:
-    def __init__(self, skoolfile, elements='btdrmsc', write_hex=False, write_asm_dirs=True):
-        parser = SkoolParser(skoolfile)
+    def __init__(self, skoolfile, elements='btdrmsc', write_hex=False, write_asm_dirs=True, preserve_base=False):
+        parser = SkoolParser(skoolfile, preserve_base)
         self.entries = parser.memory_map
         self.elements = elements
         self.write_hex = write_hex
@@ -157,7 +158,7 @@ class CtlWriter:
                             comment_text = comment.text
                             if comment.rowspan > 1 and not comment_text.replace('.', ''):
                                 comment_text = '.' + comment_text
-                        if self.need_sub_block(ctl, entry_ctl, comment_text, j, instructions):
+                        if comment_text or ctl != entry_ctl or ctl != 'c':
                             self.write_sub_block(ctl, entry_ctl, comment_text, instructions, length)
 
     def addr_str(self, address, pad=True):
@@ -167,29 +168,6 @@ class CtlWriter:
             return '{:05d}'.format(address)
         return str(address)
 
-    def need_sub_block(self, ctl, entry_ctl, comment, index, instructions):
-        # Return whether a ctl line needs to be written for a sub-block
-        if ctl == entry_ctl and not comment:
-            # Don't write a ctl line for a commentless sub-block of a code
-            # block
-            if ctl == 'c':
-                return False
-            if index == 0 and len(instructions) == 1:
-                # Don't write a ctl line for a DEF{B,M,W} statement if it's 1
-                # byte or 1 word, commentless, the first instruction in the
-                # section, the sole instruction in its sub-block, and of the
-                # same type as the containing block (i.e. 'b', 't' or 'w')
-                if ctl in 'bt' and instructions[0].size == 1:
-                    return False
-                if ctl == 'w' and instructions[0].size == 2:
-                    return False
-                # Don't write a ctl line for a DEFS statement if it's
-                # commentless, the first instruction in the section, the sole
-                # instruction in its sub-block, and inside a 'z' block
-                if ctl == 'z':
-                    return False
-        return True
-
     def get_sub_blocks(self, instructions):
         # Split a block of instructions into sub-blocks by comment rowspan
         # and/or instruction type
@@ -240,25 +218,17 @@ class CtlWriter:
         address = self.addr_str(instructions[0].address)
         lengths = ''
 
-        if ctl in 'btwz':
-            # Find the byte lengths of each line in a 'B', 'T', 'W' or 'Z'
+        if ctl in 'bstwz':
+            # Find the byte lengths of each line in a 'B', 'S', 'T', 'W' or 'Z'
             # sub-block
-            num_bytes = 0
+            length = 0
             stmt_lengths = []
             for stmt in instructions:
-                num_bytes += stmt.size
-                if ctl in 'bt':
-                    stmt_lengths.append(stmt.length)
-                else:
-                    stmt_lengths.append(str(stmt.size))
+                length += stmt.size
+                stmt_lengths.append(stmt.length)
             while len(stmt_lengths) > 1 and stmt_lengths[-1] == stmt_lengths[-2]:
                 stmt_lengths.pop()
-            # Don't write the statement lengths in a ctl line for a lone DEFB
-            # statement of length 1, a lone DEFM statement of length 1, a lone
-            # DEFW statement of length 2, or a lone DEFS statement
-            if not (len(instructions) == 1 and ((ctl in 'bt' and num_bytes == 1) or (ctl == 'w' and num_bytes == 2) or ctl == 'z')):
-                length = num_bytes
-                lengths = ',' + get_lengths(stmt_lengths)
+            lengths = ',' + get_lengths(stmt_lengths)
 
         if length:
             write_line('{0} {1},{2}{3} {4}'.format(sub_block_ctl, address, length, lengths, comment).rstrip())
@@ -266,8 +236,9 @@ class CtlWriter:
             write_line('{0} {1} {2}'.format(sub_block_ctl, address, comment).rstrip())
 
 class SkoolParser:
-    def __init__(self, skoolfile):
+    def __init__(self, skoolfile, preserve_base):
         self.skoolfile = skoolfile
+        self.preserve_base = preserve_base
         self.mode = Mode()
         self.memory_map = []
         self.header = []
@@ -395,7 +366,7 @@ class SkoolParser:
             comment_index = len(line)
         operation = line[7:comment_index].strip()
         address_comment = line[comment_index + 1:].strip()
-        instruction = Instruction(ctl + addr_str, address, operation)
+        instruction = Instruction(ctl + addr_str, address, operation, self.preserve_base)
         self.mode.apply_instruction_asm_directives(instruction)
         return instruction, address_comment
 
@@ -470,7 +441,7 @@ class FakeInstruction:
         self.asm_directives = ()
 
 class Instruction:
-    def __init__(self, label, address, operation):
+    def __init__(self, label, address, operation, preserve_base):
         self.label = label
         self.ctl = label[0]
         if label[1].isdigit():
@@ -486,13 +457,13 @@ class Instruction:
         self.size = None
         self.length = None
         if self.inst_ctl == 'b':
-            self.size, self.length = get_defb_length(operation[5:])
+            self.size, self.length = get_defb_length(operation[5:], preserve_base)
         elif self.inst_ctl == 't':
-            self.size, self.length = get_defm_length(operation[5:])
+            self.size, self.length = get_defm_length(operation[5:], preserve_base)
         elif self.inst_ctl == 'w':
-            self.size = get_defw_length(operation[5:])
-        elif self.inst_ctl == 'z':
-            self.size = parse_int(get_address(operation[4:]))
+            self.size, self.length = get_defw_length(operation[5:], preserve_base)
+        elif self.inst_ctl in 'sz':
+            self.size, self.length = get_defs_length(operation[5:], preserve_base)
 
     def set_comment(self, rowspan, text):
         self.comment = Comment(rowspan, text)
