@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2008-2013 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2008-2014 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -175,6 +175,10 @@ class HtmlWriter:
         link_operands = self.game_vars.get('LinkOperands', 'CALL,DEFW,DJNZ,JP,JR')
         self.link_operands = tuple(op.upper() for op in link_operands.split(','))
         self.game_vars.setdefault('StyleSheet', 'skoolkit.css')
+        self.js_files = ()
+        global_js = self.game_vars.get('JavaScript')
+        if global_js:
+            self.js_files = tuple(global_js.split(';'))
 
         self.bugs = self.get_sections('Bug', True)
         self.facts = self.get_sections('Fact', True)
@@ -401,7 +405,7 @@ class HtmlWriter:
             'EntryTypes': 't'
         }
         memory_maps[P_UNUSED_MAP] = {
-            'EntryTypes': 'uz',
+            'EntryTypes': 'suz',
             'PageByteColumns': '1'
         }
         return memory_maps
@@ -760,7 +764,7 @@ class HtmlWriter:
         if entry.is_routine():
             title_format = "Routine at {0}"
             page_header = page_header or "Routines"
-        elif entry.ctl in 'uz':
+        elif entry.ctl in 'suz':
             title_format = "Unused RAM at {0}"
             page_header = page_header or "Unused"
         elif entry.ctl == 'g':
@@ -794,7 +798,7 @@ class HtmlWriter:
         table_class = 'disassembly'
         comment_class = 'comment'
         transparent_class = 'transparentComment'
-        if entry.ctl not in 'cuz':
+        if entry.ctl not in 'csuz':
             table_class = 'dataDisassembly'
             comment_class = 'dataComment'
             transparent_class = 'transparentDataComment'
@@ -918,7 +922,7 @@ class HtmlWriter:
         else:
             map_name = map_details['Name']
             title = self.titles.get(map_name, map_name)
-        entry_types = map_details.get('EntryTypes', 'bcgtuwz')
+        entry_types = map_details.get('EntryTypes', 'bcgstuwz')
         show_page_byte = map_details.get('PageByteColumns', '0') != '0'
         intro = map_details.get('Intro')
         asm_path = map_details.get('AsmPath', self.code_path)
@@ -952,7 +956,7 @@ class HtmlWriter:
             elif entry.ctl in 'g':
                 entry_class = 'gbuffer'
                 desc_class = 'gbufferDesc'
-            elif entry.ctl in 'uz':
+            elif entry.ctl in 'suz':
                 size = entry.size
                 if size > 1:
                     suffix = 's'
@@ -1002,9 +1006,11 @@ class HtmlWriter:
         ofile.write('<title>{0}: {1}</title>\n'.format(self.game, title))
         for css_file in self.game_vars['StyleSheet'].split(';'):
             ofile.write('<link rel="stylesheet" type="text/css" href="{0}" />\n'.format(FileInfo.relpath(cwd, join(self.paths['StyleSheetPath'], basename(css_file)))))
+        js_files = self.js_files
         if js:
-            for js_file in js.split(';'):
-                ofile.write('<script type="text/javascript" src="{0}"></script>\n'.format(FileInfo.relpath(cwd, join(self.paths['JavaScriptPath'], basename(js_file)))))
+            js_files = list(js_files) + js.split(';')
+        for js_file in js_files:
+            ofile.write('<script type="text/javascript" src="{}"></script>\n'.format(FileInfo.relpath(cwd, join(self.paths['JavaScriptPath'], basename(js_file)))))
         ofile.write('</head>\n')
 
     def _get_logo(self, cwd):
@@ -1636,9 +1642,11 @@ class Frame(object):
     :param x: The x-coordinate of the top-left pixel to include in the frame.
     :param y: The y-coordinate of the top-left pixel to include in the frame.
     :param width: The width of the frame; if `None`, the maximum width
-                  (derived from `x` and width of the array of tiles) is used.
+                  (derived from `x` and the width of the array of tiles) is
+                  used.
     :param height: The height of the frame; if `None`, the maximum height
-                   (derived from `y` and height of the array of tiles) is used.
+                   (derived from `y` and the height of the array of tiles) is
+                   used.
     :param delay: The delay between this frame and the next in 1/100ths of a
                   second.
     """
