@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2008-2013 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2008-2014 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -22,7 +22,7 @@ import re
 from . import warn, wrap, get_int_param, parse_int, open_file, SkoolParsingError
 from .skoolmacro import DELIMITERS
 
-DIRECTIVES = 'bcgituwz'
+DIRECTIVES = 'bcgistuwz'
 
 TABLE_MARKER = '#TABLE'
 TABLE_END_MARKER = 'TABLE#'
@@ -40,14 +40,34 @@ BASE_10 = 10
 #: Force hexadecimal.
 BASE_16 = 16
 
+FORMAT_NO_BASE = {
+    'b': 'b{}',
+    'd': '{}',
+    'h': '{}'
+}
+
+FORMAT_DEFM_NO_BASE = {
+    'b': 'b{}',
+    'd': 'B{}',
+    'h': 'B{}'
+}
+
+FORMAT_PRESERVE_BASE = {
+    'b': 'b{}',
+    'd': 'd{}',
+    'h': 'h{}'
+}
+
 def get_address(operation, check_prefix=True):
     if check_prefix:
         prefixes = '[ ,(+-]'
     else:
         prefixes = ''
-    search = re.search('{0}\$[0-9A-Fa-f]+'.format(prefixes), operation)
+    search = re.search('{}\$[0-9A-Fa-f]+'.format(prefixes), operation)
     if not search:
-        search = re.search('{0}[0-9]+'.format(prefixes), operation)
+        search = re.search('{}%[01]+'.format(prefixes), operation)
+    if not search:
+        search = re.search('{}[0-9]+'.format(prefixes), operation)
     if search:
         if check_prefix:
             index = 1
@@ -64,25 +84,36 @@ def get_instruction_ctl(op):
     if op.startswith('DEFM'):
         return 'T'
     if op.startswith('DEFS'):
-        return 'Z'
+        return 'S'
     return 'C'
 
-def get_defb_length(item_str, defb=True):
+def _get_base(item, preserve_base):
+    if item.startswith('%'):
+        return 'b'
+    if item.startswith('$') and preserve_base:
+        return 'h'
+    return 'd'
+
+def get_defb_length(item_str, preserve_base, defb=True):
     if defb:
-        byte_fmt = '{0}'
-        text_fmt = 'T{0}'
+        byte_fmt = FORMAT_NO_BASE
+        text_fmt = 'T{}'
     else:
-        byte_fmt = 'B{0}'
-        text_fmt = '{0}'
+        byte_fmt = FORMAT_DEFM_NO_BASE
+        text_fmt = '{}'
+    if preserve_base:
+        byte_fmt = FORMAT_PRESERVE_BASE
     full_length = 0
     lengths = []
     length = 0
+    prev_base = None
     for item in get_defb_item_list(item_str) + ['""']:
         if item.startswith('"'):
             if length:
-                lengths.append(byte_fmt.format(length))
+                lengths.append(byte_fmt[prev_base].format(length))
                 full_length += length
                 length = 0
+                prev_base = None
             i = 1
             while i < len(item) - 1:
                 if item[i] == '\\':
@@ -94,11 +125,17 @@ def get_defb_length(item_str, defb=True):
                 full_length += length
                 length = 0
         else:
+            cur_base = _get_base(item, preserve_base)
+            if prev_base != cur_base and length:
+                lengths.append(byte_fmt[prev_base].format(length))
+                full_length += length
+                length = 0
             length += 1
+            prev_base = cur_base
     return full_length, ':'.join(lengths)
 
-def get_defm_length(item_str):
-    return get_defb_length(item_str, False)
+def get_defm_length(item_str, preserve_base):
+    return get_defb_length(item_str, preserve_base, False)
 
 def get_defb_item_list(item_str):
     items = []
@@ -130,8 +167,41 @@ def get_defb_item_list(item_str):
             i = end + 1
     return items
 
-def get_defw_length(item_str):
-    return 2 * (item_str.count(',') + 1)
+def get_defw_length(item_str, preserve_base):
+    if preserve_base:
+        word_fmt = FORMAT_PRESERVE_BASE
+    else:
+        word_fmt = FORMAT_NO_BASE
+    full_length = 0
+    lengths = []
+    length = 0
+    prev_base = None
+    for item in item_str.split(','):
+        item = item.strip()
+        cur_base = _get_base(item, preserve_base)
+        if prev_base != cur_base and length:
+            lengths.append(word_fmt[prev_base].format(length))
+            full_length += length
+            length = 0
+        length += 2
+        prev_base = cur_base
+    lengths.append(word_fmt[prev_base].format(length))
+    full_length += length
+    return full_length, ':'.join(lengths)
+
+def get_defs_length(item_str, preserve_base):
+    if preserve_base:
+        fmt = FORMAT_PRESERVE_BASE
+    else:
+        fmt = FORMAT_NO_BASE
+    size = None
+    lengths = []
+    for item in item_str.split(',', 1):
+        if size is None:
+            size = get_int_param(item)
+        base = _get_base(item, preserve_base)
+        lengths.append(fmt[base].format(item))
+    return size, ':'.join(lengths)
 
 def set_bytes(snapshot, address, operation):
     directive = operation[:4].upper()
@@ -883,7 +953,7 @@ class Mode:
 
     def replace_number(self, text, digits, check_prefix):
         num_str = get_address(text, check_prefix)
-        if num_str is None:
+        if num_str is None or num_str.startswith('%'):
             return text
         num = parse_int(num_str)
         if self.decimal:
