@@ -4,7 +4,7 @@ import unittest
 from skoolkittest import SkoolKitTestCase
 from skoolkit.skoolctl import CtlWriter
 
-DIRECTIVES = 'bcgituwz'
+DIRECTIVES = 'bcgistuwz'
 
 TEST_SKOOL = r"""; Dangling comment not associated with any entry
 
@@ -57,7 +57,7 @@ c32768 NOP          ; Do nothing
 ; @keep
 ; @ignoreua
  32779 DEFW 12345   ; W sub-block
- 32781 defs 2       ; Z sub-block
+ 32781 defs 2       ; S sub-block
 ; @nolabel
  32783 LD A,5       ; {Sub-block with instructions of various types
 ; @ofix=DEFB 2
@@ -93,20 +93,20 @@ u49157 DEFB 128
 w49158 DEFW 2
  49160 DEFW 4
 
-; Data block beginning with an implicit 1-byte sub-block
+; Data block beginning with a 1-byte sub-block
 b49162 DEFB 0
  49163 RET
 
-; Text block beginning with an implicit 1-byte sub-block
+; Text block beginning with a 1-byte sub-block
 t49164 DEFM "a"
  49165 RET
 
-; Word block beginning with an implicit 2-byte sub-block
+; Word block beginning with a 2-byte sub-block
 w49166 DEFW 23
  49168 RET
 
-; Zero block beginning with an implicit sub-block
-z49169 DEFS 9
+; Zero block
+s49169 DEFS 9
  49178 RET
 
 ; Data block with sub-block lengths amenable to abbreviation (2*3,3*2,1)
@@ -126,7 +126,7 @@ c49193 NOP
 ; @bfix+end
 
 ; Zero block
-z49194 DEFS 128
+s49194 DEFS 128
  49322 DEFS 128
  49450 DEFS 128
 ; @end
@@ -199,42 +199,49 @@ D 32774 Mid-block comment
 T 32774,5,5 T sub-block
 ; @keep:32779
 ; @ignoreua:32779
-W 32779,2 W sub-block
-Z 32781,2 Z sub-block
+W 32779,2,2 W sub-block
+S 32781,2,2 S sub-block
 M 32783,12 Sub-block with instructions of various types and blank lines in the comment
 ; @nolabel:32783
 ; @ofix:32785=DEFB 2
-B 32785,1
+B 32785,1,1
 ; @rsub:32786=DEFW 0,1,2
 W 32786,4,4
 ; @ssub:32790=DEFM "Lo"
 T 32790,2,1:B1
-Z 32792,3
+S 32792,3,3
   32795 Instruction with a comment continuation line
 E 32768 End comment paragraph 1.
 E 32768 End comment paragraph 2.
 i 32796 Ignore block
 b 49152 Data block
 ; @rem:49152=Hello!
+  49152,1,1
 g 49153 Game status buffer entry
-W 49153
+W 49153,2,2
 t 49155 Message
   49155,2,2
 u 49157 Unused block
+  49157,1,1
 w 49158 Word block
   49158,4,2
-b 49162 Data block beginning with an implicit 1-byte sub-block
+b 49162 Data block beginning with a 1-byte sub-block
+  49162,1,1
 C 49163
-t 49164 Text block beginning with an implicit 1-byte sub-block
+t 49164 Text block beginning with a 1-byte sub-block
+  49164,1,1
 C 49165
-w 49166 Word block beginning with an implicit 2-byte sub-block
+w 49166 Word block beginning with a 2-byte sub-block
+  49166,2,2
 C 49168
-z 49169 Zero block beginning with an implicit sub-block
+s 49169 Zero block
+  49169,9,9
 C 49178
 b 49179 Data block with sub-block lengths amenable to abbreviation (2*3,3*2,1)
   49179,14,2*3,3*2,1
 b 49193 ASM block directives
-z 49194 Zero block
+  49193,1,1
+s 49194 Zero block
   49194,384,128
 ; @end:49194
 b 49578 Complex DEFB statements
@@ -283,42 +290,49 @@ D $8006 Mid-block comment
 T $8006,5,5 T sub-block
 ; @keep:$800B
 ; @ignoreua:$800B
-W $800B,2 W sub-block
-Z $800D,2 Z sub-block
+W $800B,2,2 W sub-block
+S $800D,2,2 S sub-block
 M $800F,12 Sub-block with instructions of various types and blank lines in the comment
 ; @nolabel:$800F
 ; @ofix:$8011=DEFB 2
-B $8011,1
+B $8011,1,1
 ; @rsub:$8012=DEFW 0,1,2
 W $8012,4,4
 ; @ssub:$8016=DEFM "Lo"
 T $8016,2,1:B1
-Z $8018,3
+S $8018,3,3
   $801B Instruction with a comment continuation line
 E $8000 End comment paragraph 1.
 E $8000 End comment paragraph 2.
 i $801C Ignore block
 b $C000 Data block
 ; @rem:$C000=Hello!
+  $C000,1,1
 g $C001 Game status buffer entry
-W $C001
+W $C001,2,2
 t $C003 Message
   $C003,2,2
 u $C005 Unused block
+  $C005,1,1
 w $C006 Word block
   $C006,4,2
-b $C00A Data block beginning with an implicit 1-byte sub-block
+b $C00A Data block beginning with a 1-byte sub-block
+  $C00A,1,1
 C $C00B
-t $C00C Text block beginning with an implicit 1-byte sub-block
+t $C00C Text block beginning with a 1-byte sub-block
+  $C00C,1,1
 C $C00D
-w $C00E Word block beginning with an implicit 2-byte sub-block
+w $C00E Word block beginning with a 2-byte sub-block
+  $C00E,2,2
 C $C010
-z $C011 Zero block beginning with an implicit sub-block
+s $C011 Zero block
+  $C011,9,9
 C $C01A
 b $C01B Data block with sub-block lengths amenable to abbreviation (2*3,3*2,1)
   $C01B,14,2*3,3*2,1
 b $C029 ASM block directives
-z $C02A Zero block
+  $C029,1,1
+s $C02A Zero block
   $C02A,384,128
 ; @end:$C02A
 b $C1AA Complex DEFB statements
@@ -347,33 +361,40 @@ c 32768
 B 32769,2,2
 B 32771,3,1,2
 T 32774,5,5
-W 32779,2
-Z 32781,2
-B 32785,1
+W 32779,2,2
+S 32781,2,2
+B 32785,1,1
 W 32786,4,4
 T 32790,2,1:B1
-Z 32792,3
+S 32792,3,3
 i 32796
 b 49152
+  49152,1,1
 g 49153
-W 49153
+W 49153,2,2
 t 49155
   49155,2,2
 u 49157
+  49157,1,1
 w 49158
   49158,4,2
 b 49162
+  49162,1,1
 C 49163
 t 49164
+  49164,1,1
 C 49165
 w 49166
+  49166,2,2
 C 49168
-z 49169
+s 49169
+  49169,9,9
 C 49178
 b 49179
   49179,14,2*3,3*2,1
 b 49193
-z 49194
+  49193,1,1
+s 49194
   49194,384,128
 b 49578
   49578,20,3:T5:2,T7:3
@@ -395,35 +416,42 @@ c 32768
 B 32769,2,2 1-line B sub-block
 B 32771,3,1,2 2-line B sub-block
 T 32774,5,5 T sub-block
-W 32779,2 W sub-block
-Z 32781,2 Z sub-block
+W 32779,2,2 W sub-block
+S 32781,2,2 S sub-block
 M 32783,12 Sub-block with instructions of various types and blank lines in the comment
-B 32785,1
+B 32785,1,1
 W 32786,4,4
 T 32790,2,1:B1
-Z 32792,3
+S 32792,3,3
   32795 Instruction with a comment continuation line
 i 32796
 b 49152
+  49152,1,1
 g 49153
-W 49153
+W 49153,2,2
 t 49155
   49155,2,2
 u 49157
+  49157,1,1
 w 49158
   49158,4,2
 b 49162
+  49162,1,1
 C 49163
 t 49164
+  49164,1,1
 C 49165
 w 49166
+  49166,2,2
 C 49168
-z 49169
+s 49169
+  49169,9,9
 C 49178
 b 49179
   49179,14,2*3,3*2,1
 b 49193
-z 49194
+  49193,1,1
+s 49194
   49194,384,128
 b 49578
   49578,20,3:T5:2,T7:3
@@ -440,28 +468,74 @@ c 49635
   49642 ...until the end
 i 49643""".split('\n')
 
+TEST_BYTE_FORMATS_SKOOL = """; Binary and mixed-base DEFB/DEFM statements
+b30000 DEFB %10111101,$42,26
+ 30003 DEFB $38,$39,%11110000,%00001111,24,25,26
+ 30010 DEFB %11111111,%10000001
+ 30012 DEFB 47,34,56
+ 30015 DEFB $1A,$00,$00,$00,$A2
+ 30020 DEFB "hello"
+ 30025 DEFB %10101010,"hi",24,$56
+ 30030 DEFM %10111101,$42,26
+ 30033 DEFM $38,$39,%11110000,%00001111,24,25,26
+ 30040 DEFM %11111111,%10000001
+ 30042 DEFM 47,34,56
+ 30045 DEFM $1A,$00,$00,$00,$A2
+ 30050 DEFM "hello"
+ 30055 DEFM %10101010,"hi",24,$56
+"""
+
+TEST_WORD_FORMATS_SKOOL = """; Binary and mixed-base DEFW statements
+w40000 DEFW %1111000000001111,%1111000000001111
+ 40004 DEFW 12345,12345,12345
+ 40010 DEFW $AB0C,$CD32,$102F,$0000
+ 40018 DEFW 54321,%1010101010101010,$F001
+ 40024 DEFW $1234,$543C,%1111111100000000,2345,9876
+ 40034 DEFW $2345,$876D,%1001001010001011,3456,8765
+ 40044 DEFW 65535,65534
+ 40048 DEFW 1,2
+ 40052 DEFW $0000,$FFFF
+ 40056 DEFW $1000,$2FFF
+ 40060 DEFW %0101010111110101,%1111111111111111
+ 40064 DEFW %1101010111110101,%0000000000000001
+"""
+
+TEST_S_DIRECTIVES_SKOOL = """; DEFS statements in various bases
+s50000 DEFS %0000000111110100
+ 50500 DEFS 1000
+ 51500 DEFS $07D0
+ 53500 DEFS 500,%10101010
+ 54000 DEFS $0100,170
+"""
+
+TEST_Z_DIRECTIVES_SKOOL = """; DEFS statements in various bases
+z50000 DEFS %0000000111110100
+ 50500 DEFS 1000
+ 51500 DEFS $07D0
+"""
+
 class CtlWriterTest(SkoolKitTestCase):
-    def _get_ctl(self, elements='btdrmsc', write_hex=False, write_asm_dirs=True):
-        skoolfile = self.write_text_file(TEST_SKOOL, suffix='.skool')
-        writer = CtlWriter(skoolfile, elements, write_hex, write_asm_dirs)
+    def _get_ctl(self, elements='btdrmsc', write_hex=False, write_asm_dirs=True, skool=TEST_SKOOL, preserve_base=False):
+        skoolfile = self.write_text_file(skool, suffix='.skool')
+        writer = CtlWriter(skoolfile, elements, write_hex, write_asm_dirs, preserve_base)
         writer.write()
         return self.out.getvalue().split('\n')[:-1]
 
     def test_default_elements(self):
-        self.assertEqual(self._get_ctl(), TEST_CTL)
+        self.assertEqual(TEST_CTL, self._get_ctl())
 
     def test_default_elements_hex(self):
-        self.assertEqual(self._get_ctl(write_hex=True), TEST_CTL_HEX)
+        self.assertEqual(TEST_CTL_HEX, self._get_ctl(write_hex=True))
 
     def test_default_elements_no_asm_dirs(self):
         ctl = self._get_ctl(write_asm_dirs=False)
         test_ctl = [line for line in TEST_CTL if not line.startswith(';')]
-        self.assertEqual(ctl, test_ctl)
+        self.assertEqual(test_ctl, ctl)
 
     def test_wb(self):
         ctl = self._get_ctl('b', write_asm_dirs=False)
         test_ctl = [line[:7] for line in TEST_CTL if line[0] in DIRECTIVES]
-        self.assertEqual(ctl, test_ctl)
+        self.assertEqual(test_ctl, ctl)
 
     def test_wbd(self):
         ctl = self._get_ctl('bd', write_asm_dirs=False)
@@ -471,7 +545,7 @@ class CtlWriterTest(SkoolKitTestCase):
                 test_ctl.append(line[:7])
             elif line.startswith('D 32768'):
                 test_ctl.append(line)
-        self.assertEqual(ctl, test_ctl)
+        self.assertEqual(test_ctl, ctl)
 
     def test_wbm(self):
         ctl = self._get_ctl('bm', write_asm_dirs=False)
@@ -481,7 +555,7 @@ class CtlWriterTest(SkoolKitTestCase):
                 test_ctl.append(line[:7])
             elif line[0] in 'DE' and not line.startswith('D 32768'):
                 test_ctl.append(line)
-        self.assertEqual(ctl, test_ctl)
+        self.assertEqual(test_ctl, ctl)
 
     def test_wbr(self):
         ctl = self._get_ctl('br', write_asm_dirs=False)
@@ -491,20 +565,86 @@ class CtlWriterTest(SkoolKitTestCase):
                 test_ctl.append(line[:7])
             elif line[0] == 'R':
                 test_ctl.append(line)
-        self.assertEqual(ctl, test_ctl)
+        self.assertEqual(test_ctl, ctl)
 
     def test_wbs(self):
         ctl = self._get_ctl('bs', write_asm_dirs=False)
-        self.assertEqual(ctl, TEST_CTL_BS)
+        self.assertEqual(TEST_CTL_BS, ctl)
 
     def test_wbsc(self):
         ctl = self._get_ctl('bsc', write_asm_dirs=False)
-        self.assertEqual(ctl, TEST_CTL_BSC)
+        self.assertEqual(TEST_CTL_BSC, ctl)
 
     def test_wbt(self):
         ctl = self._get_ctl('bt', write_asm_dirs=False)
         test_ctl = [line for line in TEST_CTL if line[0] in DIRECTIVES]
-        self.assertEqual(ctl, test_ctl)
+        self.assertEqual(test_ctl, ctl)
+
+    def test_byte_formats_no_base(self):
+        ctl = self._get_ctl(skool=TEST_BYTE_FORMATS_SKOOL, preserve_base=False)
+        exp_ctl = [
+            'b 30000 Binary and mixed-base DEFB/DEFM statements',
+            '  30000,30,b1:2,2:b2:3,b2,3,5,T5,b1:T2:2',
+            'T 30030,30,b1:B2,B2:b2:B3,b2,B3,B5,5,b1:2:B2'
+        ]
+        self.assertEqual(exp_ctl, ctl)
+
+    def test_byte_formats_preserve_base(self):
+        ctl = self._get_ctl(skool=TEST_BYTE_FORMATS_SKOOL, preserve_base=True)
+        exp_ctl = [
+            'b 30000 Binary and mixed-base DEFB/DEFM statements',
+            '  30000,30,b1:h1:d1,h2:b2:d3,b2,d3,h5,T5,b1:T2:d1:h1',
+            'T 30030,30,b1:h1:d1,h2:b2:d3,b2,d3,h5,5,b1:2:d1:h1'
+        ]
+        self.assertEqual(exp_ctl, ctl)
+
+    def test_word_formats_no_base(self):
+        ctl = self._get_ctl(skool=TEST_WORD_FORMATS_SKOOL, preserve_base=False)
+        exp_ctl = [
+            'w 40000 Binary and mixed-base DEFW statements',
+            '  40000,68,b4,6,8,2:b2:2,4:b2:4*2,4*4,b4'
+        ]
+        self.assertEqual(exp_ctl, ctl)
+
+    def test_word_formats_preserve_base(self):
+        ctl = self._get_ctl(skool=TEST_WORD_FORMATS_SKOOL, preserve_base=True)
+        exp_ctl = [
+            'w 40000 Binary and mixed-base DEFW statements',
+            '  40000,68,b4,d6,h8,d2:b2:h2,h4:b2:d4*2,d4*2,h4*2,b4'
+        ]
+        self.assertEqual(exp_ctl, ctl)
+
+    def test_s_directives_no_base(self):
+        ctl = self._get_ctl(skool=TEST_S_DIRECTIVES_SKOOL, preserve_base=False)
+        exp_ctl = [
+            's 50000 DEFS statements in various bases',
+            '  50000,4256,b%0000000111110100,1000,$07D0,500:b%10101010,$0100:170'
+        ]
+        self.assertEqual(exp_ctl, ctl)
+
+    def test_s_directives_preserve_base(self):
+        ctl = self._get_ctl(skool=TEST_S_DIRECTIVES_SKOOL, preserve_base=True)
+        exp_ctl = [
+            's 50000 DEFS statements in various bases',
+            '  50000,4256,b%0000000111110100,d1000,h$07D0,d500:b%10101010,h$0100:d170'
+        ]
+        self.assertEqual(exp_ctl, ctl)
+
+    def test_z_directives_no_base(self):
+        ctl = self._get_ctl(skool=TEST_Z_DIRECTIVES_SKOOL, preserve_base=False)
+        exp_ctl = [
+            'z 50000 DEFS statements in various bases',
+            'S 50000,3500,b%0000000111110100,1000,$07D0'
+        ]
+        self.assertEqual(exp_ctl, ctl)
+
+    def test_z_directives_preserve_base(self):
+        ctl = self._get_ctl(skool=TEST_Z_DIRECTIVES_SKOOL, preserve_base=True)
+        exp_ctl = [
+            'z 50000 DEFS statements in various bases',
+            'S 50000,3500,b%0000000111110100,d1000,h$07D0'
+        ]
+        self.assertEqual(exp_ctl, ctl)
 
 if __name__ == '__main__':
     unittest.main()
