@@ -65,10 +65,10 @@ u 32797
 ; Entry #11
 u 32798 Unimportant unused byte
 ; Entry #12
-z 32799
-Z 32799,10,5
+s 32799
+S 32799,10,5
 ; Entry #13
-z 32809 Block of zeroes
+s 32809 Block of zeroes
 M 32809,5 A DEFS followed by two NOPs
 C 32812,2
 ; Entry #14
@@ -355,6 +355,316 @@ class DisassemblyTest(SkoolKitTestCase):
         entry = entries[15]
         self.assertEqual(entry.address, 32837)
 
+    def test_byte_formats(self):
+        snapshot = [42] * 75
+        ctl = '\n'.join((
+            'b 00000',
+            '  00000,b5',
+            '  00005,b15',
+            '  00020,b5,2,d2,h1',
+            'B 00025,b5,2:d2:h1',
+            '  00030,h10,5:d3:b2',
+            'B 00040,5,b1,h2',
+            '  00045,5,h1,T4',
+            '  00050,5,b2:T3',
+            'T 00055,5,h2,3',
+            'T 00060,5,2:d3',
+            'T 00065,5,3,B1',
+            'T 00070,5,B2:h3',
+            'i 00075'
+        ))
+        ctl_parser = CtlParser()
+        ctl_parser.parse_ctl(self.write_text_file(ctl))
+        disassembly = Disassembly(snapshot, ctl_parser, True)
+
+        entries = disassembly.entries
+        self.assertEqual(len(entries), 2)
+
+        entry = entries[0]
+        self.assertEqual(entry.address, 0)
+        instructions = entry.instructions
+        actual_instructions = [(i.address, i.operation) for i in instructions]
+        exp_instructions = [
+            ( 0, 'DEFB %00101010,%00101010,%00101010,%00101010,%00101010'),
+            ( 5, 'DEFB %00101010,%00101010,%00101010,%00101010,%00101010,%00101010,%00101010,%00101010'),
+            (13, 'DEFB %00101010,%00101010,%00101010,%00101010,%00101010,%00101010,%00101010'),
+            (20, 'DEFB %00101010,%00101010'),
+            (22, 'DEFB 42,42'),
+            (24, 'DEFB $2A'),
+            (25, 'DEFB %00101010,%00101010,42,42,$2A'),
+            (30, 'DEFB $2A,$2A,$2A,$2A,$2A,42,42,42,%00101010,%00101010'),
+            (40, 'DEFB %00101010'),
+            (41, 'DEFB $2A,$2A'),
+            (43, 'DEFB $2A,$2A'),
+            (45, 'DEFB $2A'),
+            (46, 'DEFB "****"'),
+            (50, 'DEFB %00101010,%00101010,"***"'),
+            (55, 'DEFM $2A,$2A'),
+            (57, 'DEFM "***"'),
+            (60, 'DEFM "**",42,42,42'),
+            (65, 'DEFM "***"'),
+            (68, 'DEFM 42'),
+            (69, 'DEFM 42'),
+            (70, 'DEFM 42,42,$2A,$2A,$2A')
+        ]
+        self.assertEqual(actual_instructions, exp_instructions)
+
+    def test_byte_formats_hex(self):
+        snapshot = [85] * 4
+        ctl = '\n'.join((
+            'b 00000',
+            '  00000,4,b1:d1:h1:1',
+            'i 00004'
+        ))
+        ctl_parser = CtlParser()
+        ctl_parser.parse_ctl(self.write_text_file(ctl))
+        disassembly = Disassembly(snapshot, ctl_parser, True, asm_hex=True)
+
+        entries = disassembly.entries
+        self.assertEqual(len(entries), 2)
+
+        entry = entries[0]
+        self.assertEqual(entry.address, 0)
+        instructions = entry.instructions
+        self.assertEqual(len(instructions), 1)
+        defb = instructions[0]
+        self.assertEqual(defb.operation, 'DEFB %01010101,85,$55,$55')
+
+    def test_word_formats(self):
+        snapshot = [170, 53] * 32
+        ctl = '\n'.join((
+            'w 00000',
+            '  00000,4',
+            '  00004,b4',
+            '  00008,d4',
+            'W 00012,h4',
+            'W 00016,b8,2,d2,h4',
+            '  00024,d8,b4:2:h2',
+            '  00032,h8,b2:d4:2',
+            '  00040,8,b2,4,h2',
+            'W 00048,8,b2:2:h4',
+            '  00056,8,4',
+            'i 00064'
+        ))
+        ctl_parser = CtlParser()
+        ctl_parser.parse_ctl(self.write_text_file(ctl))
+        disassembly = Disassembly(snapshot, ctl_parser, True)
+
+        entries = disassembly.entries
+        self.assertEqual(len(entries), 2)
+
+        entry = entries[0]
+        self.assertEqual(entry.address, 0)
+        instructions = entry.instructions
+        actual_instructions = [(i.address, i.operation) for i in instructions]
+        exp_instructions = [
+            ( 0, 'DEFW 13738'),
+            ( 2, 'DEFW 13738'),
+            ( 4, 'DEFW %0011010110101010'),
+            ( 6, 'DEFW %0011010110101010'),
+            ( 8, 'DEFW 13738'),
+            (10, 'DEFW 13738'),
+            (12, 'DEFW $35AA'),
+            (14, 'DEFW $35AA'),
+            (16, 'DEFW %0011010110101010'),
+            (18, 'DEFW 13738'),
+            (20, 'DEFW $35AA,$35AA'),
+            (24, 'DEFW %0011010110101010,%0011010110101010,13738,$35AA'),
+            (32, 'DEFW %0011010110101010,13738,13738,$35AA'),
+            (40, 'DEFW %0011010110101010'),
+            (42, 'DEFW 13738,13738'),
+            (46, 'DEFW $35AA'),
+            (48, 'DEFW %0011010110101010,13738,$35AA,$35AA'),
+            (56, 'DEFW 13738,13738'),
+            (60, 'DEFW 13738,13738')
+        ]
+        self.assertEqual(actual_instructions, exp_instructions)
+
+    def test_word_formats_hex(self):
+        snapshot = [240] * 8
+        ctl = '\n'.join((
+            'w 00000',
+            '  00000,8,b2,d2,h2,2',
+            'i 00008'
+        ))
+        ctl_parser = CtlParser()
+        ctl_parser.parse_ctl(self.write_text_file(ctl))
+        disassembly = Disassembly(snapshot, ctl_parser, True, asm_hex=True)
+
+        entries = disassembly.entries
+        self.assertEqual(len(entries), 2)
+
+        entry = entries[0]
+        self.assertEqual(entry.address, 0)
+        instructions = entry.instructions
+        actual_instructions = [(i.address, i.operation) for i in instructions]
+        exp_instructions = [
+            (0, 'DEFW %1111000011110000'),
+            (2, 'DEFW 61680'),
+            (4, 'DEFW $F0F0'),
+            (6, 'DEFW $F0F0')
+        ]
+        self.assertEqual(exp_instructions, actual_instructions)
+
+    def test_s_directives(self):
+        snapshot = []
+        ctl = '\n'.join((
+            's 00000',
+            '  00000,4',
+            '  00004,b4',
+            'S 00008,d4',
+            '  00012,h8',
+            '  00020,40,b10,10,h10',
+            'S 00060,b40,10,d10,h10',
+            '  00100,d40,b10,10,h10',
+            '  00140,h60,b10,d10,40',
+            'S 00200,768,b256,d256,h256',
+            '  00968,56,16:b%10101010,40:h17',
+            'i 01024'
+        ))
+        ctl_parser = CtlParser()
+        ctl_parser.parse_ctl(self.write_text_file(ctl))
+        disassembly = Disassembly(snapshot, ctl_parser, True)
+
+        entries = disassembly.entries
+        self.assertEqual(len(entries), 2)
+
+        entry = entries[0]
+        self.assertEqual(entry.address, 0)
+        instructions = entry.instructions
+        actual_instructions = [(i.address, i.operation) for i in instructions]
+        exp_instructions = [
+            (  0, 'DEFS 4'),
+            (  4, 'DEFS %00000100'),
+            (  8, 'DEFS 4'),
+            ( 12, 'DEFS 8'),
+            ( 20, 'DEFS %00001010'),
+            ( 30, 'DEFS 10'),
+            ( 40, 'DEFS $0A'),
+            ( 50, 'DEFS $0A'),
+            ( 60, 'DEFS %00001010'),
+            ( 70, 'DEFS 10'),
+            ( 80, 'DEFS $0A'),
+            ( 90, 'DEFS $0A'),
+            (100, 'DEFS %00001010'),
+            (110, 'DEFS 10'),
+            (120, 'DEFS $0A'),
+            (130, 'DEFS $0A'),
+            (140, 'DEFS %00001010'),
+            (150, 'DEFS 10'),
+            (160, 'DEFS $28'),
+            (200, 'DEFS %0000000100000000'),
+            (456, 'DEFS 256'),
+            (712, 'DEFS $0100'),
+            (968, 'DEFS 16,%10101010'),
+            (984, 'DEFS 40,$11')
+        ]
+        self.assertEqual(exp_instructions, actual_instructions)
+
+    def test_s_directives_hex(self):
+        snapshot = []
+        ctl = '\n'.join((
+            's 00000',
+            '  00000,14,d2:b1,h2:128,h10:2',
+            'i 00014'
+        ))
+        ctl_parser = CtlParser()
+        ctl_parser.parse_ctl(self.write_text_file(ctl))
+        disassembly = Disassembly(snapshot, ctl_parser, True, asm_hex=True)
+
+        entries = disassembly.entries
+        self.assertEqual(len(entries), 2)
+
+        entry = entries[0]
+        self.assertEqual(entry.address, 0)
+        instructions = entry.instructions
+        actual_instructions = [(i.address, i.operation) for i in instructions]
+        exp_instructions = [
+            (0, 'DEFS 2,%00000001'),
+            (2, 'DEFS 2,$80'),
+            (4, 'DEFS $0A,2')
+        ]
+        self.assertEqual(exp_instructions, actual_instructions)
+
+    def test_z_directives(self):
+        snapshot = []
+        ctl = '\n'.join((
+            'z 00000',
+            '  00000,4',
+            '  00004,b4',
+            'Z 00008,d4',
+            '  00012,h8',
+            '  00020,40,b10,10,h10',
+            'Z 00060,b40,10,d10,h10',
+            '  00100,d40,b10,10,h10',
+            '  00140,h60,b10,d10,40',
+            'Z 00200,768,b256,d256,h256',
+            'i 00968'
+        ))
+        ctl_parser = CtlParser()
+        ctl_parser.parse_ctl(self.write_text_file(ctl))
+        disassembly = Disassembly(snapshot, ctl_parser, True)
+
+        entries = disassembly.entries
+        self.assertEqual(len(entries), 2)
+
+        entry = entries[0]
+        self.assertEqual(entry.address, 0)
+        instructions = entry.instructions
+        actual_instructions = [(i.address, i.operation) for i in instructions]
+        exp_instructions = [
+            (  0, 'DEFS 4'),
+            (  4, 'DEFS %00000100'),
+            (  8, 'DEFS 4'),
+            ( 12, 'DEFS 8'),
+            ( 20, 'DEFS %00001010'),
+            ( 30, 'DEFS 10'),
+            ( 40, 'DEFS $0A'),
+            ( 50, 'DEFS $0A'),
+            ( 60, 'DEFS %00001010'),
+            ( 70, 'DEFS 10'),
+            ( 80, 'DEFS $0A'),
+            ( 90, 'DEFS $0A'),
+            (100, 'DEFS %00001010'),
+            (110, 'DEFS 10'),
+            (120, 'DEFS $0A'),
+            (130, 'DEFS $0A'),
+            (140, 'DEFS %00001010'),
+            (150, 'DEFS 10'),
+            (160, 'DEFS $28'),
+            (200, 'DEFS %0000000100000000'),
+            (456, 'DEFS 256'),
+            (712, 'DEFS $0100')
+        ]
+        self.assertEqual(exp_instructions, actual_instructions)
+
+    def test_z_directives_hex(self):
+        snapshot = []
+        ctl = '\n'.join((
+            'z 00000',
+            '  00000,26,b2,d2,h2,h10,10',
+            'i 00026'
+        ))
+        ctl_parser = CtlParser()
+        ctl_parser.parse_ctl(self.write_text_file(ctl))
+        disassembly = Disassembly(snapshot, ctl_parser, True, asm_hex=True)
+
+        entries = disassembly.entries
+        self.assertEqual(len(entries), 2)
+
+        entry = entries[0]
+        self.assertEqual(entry.address, 0)
+        instructions = entry.instructions
+        actual_instructions = [(i.address, i.operation) for i in instructions]
+        exp_instructions = [
+            ( 0, 'DEFS %00000010'),
+            ( 2, 'DEFS 2'),
+            ( 4, 'DEFS 2'),
+            ( 6, 'DEFS $0A'),
+            (16, 'DEFS $0A')
+        ]
+        self.assertEqual(exp_instructions, actual_instructions)
+
 class SkoolWriterTest(SkoolKitTestCase):
     def _get_writer(self, snapshot, ctl, defb_size=8, defb_mod=1, zfill=False, defm_width=66, asm_hex=False, asm_lower=False):
         ctl_parser = CtlParser()
@@ -601,13 +911,13 @@ class SkoolWriterTest(SkoolKitTestCase):
 
 class CtlWriterTest(SkoolKitTestCase):
     def test_decimal_addresses_below_10000(self):
-        ctls = {0: 'b', 1: 'c', 22: 't', 333: 'w', 4444: 'z'}
+        ctls = {0: 'b', 1: 'c', 22: 't', 333: 'w', 4444: 's'}
         exp_ctl = [
             'b 00000',
             'c 00001',
             't 00022',
             'w 00333',
-            'z 04444'
+            's 04444'
         ]
         ctlfile = self.write_bin_file()
         write_ctl(ctlfile, ctls, False)
